==> epath-location-1.2.3-sources.jar: com/shitu/location/uploadlocation/engine/model/LocationRegion.java <==
package com.shitu.location.uploadlocation.engine.model;//package com.daoyixun.uploadlocation.engine.model;
//
//import android.support.annotation.NonNull;
//
//import com.sails.engine.DijkstraAlgorithm.DijkstraNode;
//import com.sails.engine.SAILS.*;
//import com.sails.engine.core.util.MercatorProjection;
//
//import java.util.ArrayList;
//import java.util.Comparator;
//import java.util.List;
//
//
//public class LocationRegion {
//
//    public final static int NORMAL=0;
//    public final static int ENGLISH=1;
//    public final static int CHINESE_BIG5=2;
//    public final static int CHINESE_SM=3;
//    public final static int JAPANESE=4;
//    public static int FONT_LANGUAGE=CHINESE_BIG5;
//    public long id;
//	public List<GeoNode> mBoundaryVertexList = new ArrayList<GeoNode>();
//	public String label;
//    MapDataBase floor;
//    public Beacon beacon=null;
//	public int level;
//    public String chinese_t=null;
//    public String chinese_s=null;
//    public String english=null;
//    public String japanese=null;
//    public String type=null;
//    public String subtype=null;
//    public String url=null;
//    public int self=Integer.MAX_VALUE;
//    public List<Integer> goToList =  new ArrayList<Integer>();
//    public double fusionCoefficient =0;
//    boolean isUseTriLocating=false;
//    boolean gpsArea=false;
//    double centerLon,centerLat;
//
//    //updated in 2017/2/14
//    public double updatedLength=-1; //save path routing length
//    public int changeFloorCount=0;
//    boolean useDynamicPathLink=false;
//    DijkstraAlgorithm3D.Dijkstra3DNode dynamicAnchor=null;
//    List<DijkstraNode> mPathNodeList =  null;
//    public LocationRegion(){
//
//    }
//    public LocationRegion(String labelName, double longitude, double latitude, String floor,
//                          @NonNull SAILS sails) {
//        setFloor(sails,floor);
//        centerLat=latitude;
//        centerLon=longitude;
//        label=labelName;
//        setPathRoutingNode(sails);
//
//
//    }
//    void setPathRoutingNode(SAILS sails) {
//        if(hasPathNode()
//                ||useDynamicPathLink
//                ||floor==null
//                ||sails==null
//                ||sails.msails.currentBDB==null
//                ||sails.msails.currentBDB.dijkstra3DNodeList==null)
//            return;
//        dynamicAnchor=sails.msails.currentBDB.findClosestDijkstra3DNodeUsingLineMethod(
//                centerLon,
//                centerLat,
//                this.floor.floorNumber);
//
//        if(dynamicAnchor!=null&&dynamicAnchor.mPrevious!=null) {
//            dynamicAnchor=dynamicAnchor.mPrevious;
//            useDynamicPathLink = true;
//        }
//
//    }
//    void setFloor(SAILS sails,String floorName) {
//        for(MapDataBase mdb:sails.msails.currentBDB.floorLists) {
//            if(mdb.floorName.equals(floorName)) {
//                floor=mdb;
//            }
//        }
//    }
//
//    /**
//     * Get location region's floor name.
//     *
//     * @return floor name in string.
//     */
//    public String getFloorName() {
//        if(floor!=null)
//            return floor.floorName;
//        return null;
//    }
//
//    /**
//     * Get location region's floor description.
//     *
//     * @return floor name in string.
//     */
//    public String getFloorDescription() {
//        return floor.floorDesc;
//    }
//
//    /**
//     * Get location region's floor number.
//     *
//     * @return floor name in string.
//     */
//    public int getFloorNumber() {
//        if(floor!=null)
//            return floor.floorNumber;
//        return 0;
//    }
//
//    /**
//     * Get current Languaue type.
//     *
//     * @return languaue type int.
//     */
//    public String getName() {
//        switch(FONT_LANGUAGE) {
//            case NORMAL:
//                return label.replace(" - ","");
//            case CHINESE_BIG5:
//                return chinese_t;
//            case CHINESE_SM:
//                return chinese_s;
//            case ENGLISH:
//                return english;
//            case JAPANESE:
//                return japanese;
//            default:
//                return label;
//        }
//    }
//
//    /**
//     * Check if the specified location is in the location region or not.
//     *
//     * @param lon input longitude.
//     * @param lat input latitude.
//     * @return true if is in the location region.
//     */
//	public boolean isInRegion(double lon, double lat) {
//		if (mBoundaryVertexList.size() < 3)
//			return false;
//		int i, j;
//		boolean c = false;
//		for (i = 0, j = mBoundaryVertexList.size() - 1; i < mBoundaryVertexList.size(); j = i++) {
//			if (((mBoundaryVertexList.get(i).latitude > lat) != (mBoundaryVertexList.get(j).latitude > lat))
//					&& (lon < (mBoundaryVertexList.get(j).longitude - mBoundaryVertexList.get(i).longitude)
//							* (lat - mBoundaryVertexList.get(i).latitude)/ (mBoundaryVertexList.get(j).latitude - mBoundaryVertexList
//									.get(i).latitude)+ mBoundaryVertexList.get(i).longitude))
//				c = !c;
//		}
//		return c;
//	}
//
//    /**
//     * Check the region has routing node or not.
//     *
//     * @return true if region has routing nodes.
//     */
//    public boolean hasPathNode(){
//        if(mPathNodeList==null || mPathNodeList.size()==0)
//            return false;
//
//        return true;
//    }
//
//    /**
//     * Get the center longitude of the location region.
//     *
//     * @return longitude value in double.
//     */
//    public double getCenterLongitude() {
//        return centerLon;
//    }
//
//    /**
//     * Get the center latitude of the location region.
//     *
//     * @return latitude value in double.
//     */
//    public double getCenterLatitude() {
//        return centerLat;
//    }
//    /*Replace*/ public static class Vertex {
//        /*Replace*/ public final double longitude;
//        /*Replace*/ public final double latitude;
//
//        Vertex(double lon, double lat) {
//            this.longitude = lon;
//            this.latitude = lat;
//        }
//    }
//
//    /**
//     * Define location region vertex.
//     *
//     * @param vertexList List of GeoNode.
//     */
//    public void setVertexList(List<GeoNode> vertexList) {
//        if(vertexList.size()==1) {
//            centerLat=vertexList.get(0).latitude;
//            centerLon=vertexList.get(0).longitude;
//            return;
//        }
//        mBoundaryVertexList=vertexList;
//       Point[] plist=new Point[vertexList.size()+1];
//        boolean first=false;
//        int i=0;
//        Point initial=null;
//        for(GeoNode gp:vertexList) {
//            double x= MercatorProjection.longitudeToPixelX(gp.longitude,(byte)22);
//            double y= MercatorProjection.latitudeToPixelY(gp.latitude, (byte) 22);
//            double xy[]={x,y};
//            if(i==0) {
//                initial=new Point(x,y);
//            }
//            Point p=new Point(xy[0]-initial.x,xy[1]-initial.y);
//            if(i==0)
//                plist[vertexList.size()]=p;
//            plist[i]=p;
//            i++;
//        }
//        Point center=centroid(plist);
//        double lon=MercatorProjection.pixelXToLongitude(center.x+initial.x,(byte)22);
//        double lat=MercatorProjection.pixelYToLatitude(center.y+initial.y,(byte)22);
//        centerLat=lat;
//        centerLon=lon;
//
//    }
//    class Point {
//        final double x,y;
//        Point(double x, double y) {
//            this.x=x;
//            this.y=y;
//        }
//    }
//    private double signedArea(Point[] points) {
//        double sum = 0.0;
//        for (int i = 0; i < points.length-1; i++) {
//            sum = sum + (points[i].x * points[i+1].y) - (points[i].y * points[i+1].x);
//        }
//        return 0.5 * sum;
//    }
//    private Point centroid(Point[] points) {
////        double area=Math.abs(signedArea(points));
////        double cx = 0.0, cy = 0.0;
////        for (int i = 0; i < points.length-1; i++) {
////            cx = cx + (points[i].x + points[i+1].x) * (points[i].y * points[i+1].x - points[i].x * points[i+1].y);
////            cy = cy + (points[i].y + points[i+1].y) * (points[i].y * points[i+1].x - points[i].x * points[i+1].y);
////        }
////        cx /= (6 * area);
////        cy /= (6 * area);
////        return new Point(cx, cy);
//        double cx = 0.0, cy = 0.0, area=0;
//        for (int i = 0; i < points.length-1; i++) {
//            double areaSum =   -(points[i].y * points[i+1].x - points[i].x * points[i+1].y);
//            cx = cx + (points[i].x + points[i+1].x) * areaSum;//(points[i].y * points[i+1].x - points[i].x * points[i+1].y);
//            cy = cy + (points[i].y + points[i+1].y) * areaSum;//(points[i].y * points[i+1].x - points[i].x * points[i+1].y);
//            area += areaSum;
//        }
//        cx /= (6 * 0.5*area);
//        cy /= (6 * 0.5*area);
//
//        return new Point(cx, cy);
//    }
////    /*Replace*/ public void setVertexList(List<GeoNode> mGNList) {
////        mBoundaryVertexList = mGNList;
////        centerLon = 0;
////        centerLat = 0;
////        for (SAILS.GeoNode mGN : mGNList) {
////            if (mGN == mGNList.get(mGNList.size() - 1))
////                break;
////            centerLon += mGN.longitude;
////            centerLat += mGN.latitude;
////        }
////        centerLon /= mGNList.size() - 1;
////        centerLat /= mGNList.size() - 1;
////    }
//
//    static class RegionByLevelComparator implements Comparator<LocationRegion> {
//
//        @Override
//        public int compare(LocationRegion locationRegion, LocationRegion locationRegion2) {
//            if(locationRegion.level<locationRegion2.level)
//                return 1;
//            if(locationRegion.level==locationRegion2.level)
//                return 0;
//            return -1;
//        }
//    }
//}

==> epath-location-1.2.8-sources.jar: com/shitu/location/uploadlocation/engine/model/LocationRegion.java <==
package com.shitu.location.uploadlocation.engine.model;//package com.epath.uploadlocation.engine.model;
//
//import android.support.annotation.NonNull;
//
//import com.sails.engine.DijkstraAlgorithm.DijkstraNode;
//import com.sails.engine.SAILS.*;
//import com.sails.engine.core.util.MercatorProjection;
//
//import java.util.ArrayList;
//import java.util.Comparator;
//import java.util.List;
//
//
//public class LocationRegion {
//
//    public final static int NORMAL=0;
//    public final static int ENGLISH=1;
//    public final static int CHINESE_BIG5=2;
//    public final static int CHINESE_SM=3;
//    public final static int JAPANESE=4;
//    public static int FONT_LANGUAGE=CHINESE_BIG5;
//    public long id;
//	public List<GeoNode> mBoundaryVertexList = new ArrayList<GeoNode>();
//	public String label;
//    MapDataBase floor;
//    public Beacon beacon=null;
//	public int level;
//    public String chinese_t=null;
//    public String chinese_s=null;
//    public String english=null;
//    public String japanese=null;
//    public String type=null;
//    public String subtype=null;
//    public String url=null;
//    public int self=Integer.MAX_VALUE;
//    public List<Integer> goToList =  new ArrayList<Integer>();
//    public double fusionCoefficient =0;
//    boolean isUseTriLocating=false;
//    boolean gpsArea=false;
//    double centerLon,centerLat;
//
//    //updated in 2017/2/14
//    public double updatedLength=-1; //save path routing length
//    public int changeFloorCount=0;
//    boolean useDynamicPathLink=false;
//    DijkstraAlgorithm3D.Dijkstra3DNode dynamicAnchor=null;
//    List<DijkstraNode> mPathNodeList =  null;
//    public LocationRegion(){
//
//    }
//    public LocationRegion(String labelName, double longitude, double latitude, String floor,
//                          @NonNull SAILS sails) {
//        setFloor(sails,floor);
//        centerLat=latitude;
//        centerLon=longitude;
//        label=labelName;
//        setPathRoutingNode(sails);
//
//
//    }
//    void setPathRoutingNode(SAILS sails) {
//        if(hasPathNode()
//                ||useDynamicPathLink
//                ||floor==null
//                ||sails==null
//                ||sails.msails.currentBDB==null
//                ||sails.msails.currentBDB.dijkstra3DNodeList==null)
//            return;
//        dynamicAnchor=sails.msails.currentBDB.findClosestDijkstra3DNodeUsingLineMethod(
//                centerLon,
//                centerLat,
//                this.floor.floorNumber);
//
//        if(dynamicAnchor!=null&&dynamicAnchor.mPrevious!=null) {
//            dynamicAnchor=dynamicAnchor.mPrevious;
//            useDynamicPathLink = true;
//        }
//
//    }
//    void setFloor(SAILS sails,String floorName) {
//        for(MapDataBase mdb:sails.msails.currentBDB.floorLists) {
//            if(mdb.floorName.equals(floorName)) {
//                floor=mdb;
//            }
//        }
//    }
//
//    /**
//     * Get location region's floor name.
//     *
//     * @return floor name in string.
//     */
//    public String getFloorName() {
//        if(floor!=null)
//            return floor.floorName;
//        return null;
//    }
//
//    /**
//     * Get location region's floor description.
//     *
//     * @return floor name in string.
//     */
//    public String getFloorDescription() {
//        return floor.floorDesc;
//    }
//
//    /**
//     * Get location region's floor number.
//     *
//     * @return floor name in string.
//     */
//    public int getFloorNumber() {
//        if(floor!=null)
//            return floor.floorNumber;
//        return 0;
//    }
//
//    /**
//     * Get current Languaue type.
//     *
//     * @return languaue type int.
//     */
//    public String getName() {
//        switch(FONT_LANGUAGE) {
//            case NORMAL:
//                return label.replace(" - ","");
//            case CHINESE_BIG5:
//                return chinese_t;
//            case CHINESE_SM:
//                return chinese_s;
//            case ENGLISH:
//                return english;
//            case JAPANESE:
//                return japanese;
//            default:
//                return label;
//        }
//    }
//
//    /**
//     * Check if the specified location is in the location region or not.
//     *
//     * @param lon input longitude.
//     * @param lat input latitude.
//     * @return true if is in the location region.
//     */
//	public boolean isInRegion(double lon, double lat) {
//		if (mBoundaryVertexList.size() < 3)
//			return false;
//		int i, j;
//		boolean c = false;
//		for (i = 0, j = mBoundaryVertexList.size() - 1; i < mBoundaryVertexList.size(); j = i++) {
//			if (((mBoundaryVertexList.get(i).latitude > lat) != (mBoundaryVertexList.get(j).latitude > lat))
//					&& (lon < (mBoundaryVertexList.get(j).longitude - mBoundaryVertexList.get(i).longitude)
//							* (lat - mBoundaryVertexList.get(i).latitude)/ (mBoundaryVertexList.get(j).latitude - mBoundaryVertexList
//									.get(i).latitude)+ mBoundaryVertexList.get(i).longitude))
//				c = !c;
//		}
//		return c;
//	}
//
//    /**
//     * Check the region has routing node or not.
//     *
//     * @return true if region has routing nodes.
//     */
//    public boolean hasPathNode(){
//        if(mPathNodeList==null || mPathNodeList.size()==0)
//            return false;
//
//        return true;
//    }
//
//    /**
//     * Get the center longitude of the location region.
//     *
//     * @return longitude value in double.
//     */
//    public double getCenterLongitude() {
//        return centerLon;
//    }
//
//    /**
//     * Get the center latitude of the location region.
//     *
//     * @return latitude value in double.
//     */
//    public double getCenterLatitude() {
//        return centerLat;
//    }
//    /*Replace*/ public static class Vertex {
//        /*Replace*/ public final double longitude;
//        /*Replace*/ public final double latitude;
//
//        Vertex(double lon, double lat) {
//            this.longitude = lon;
//            this.latitude = lat;
//        }
//    }
//
//    /**
//     * Define location region vertex.
//     *
//     * @param vertexList List of GeoNode.
//     */
//    public void setVertexList(List<GeoNode> vertexList) {
//        if(vertexList.size()==1) {
//            centerLat=vertexList.get(0).latitude;
//            centerLon=vertexList.get(0).longitude;
//            return;
//        }
//        mBoundaryVertexList=vertexList;
//       Point[] plist=new Point[vertexList.size()+1];
//        boolean first=false;
//        int i=0;
//        Point initial=null;
//        for(GeoNode gp:vertexList) {
//            double x= MercatorProjection.longitudeToPixelX(gp.longitude,(byte)22);
//            double y= MercatorProjection.latitudeToPixelY(gp.latitude, (byte) 22);
//            double xy[]={x,y};
//            if(i==0) {
//                initial=new Point(x,y);
//            }
//            Point p=new Point(xy[0]-initial.x,xy[1]-initial.y);
//            if(i==0)
//                plist[vertexList.size()]=p;
//            plist[i]=p;
//            i++;
//        }
//        Point center=centroid(plist);
//        double lon=MercatorProjection.pixelXToLongitude(center.x+initial.x,(byte)22);
//        double lat=MercatorProjection.pixelYToLatitude(center.y+initial.y,(byte)22);
//        centerLat=lat;
//        centerLon=lon;
//
//    }
//    class Point {
//        final double x,y;
//        Point(double x, double y) {
//            this.x=x;
//            this.y=y;
//        }
//    }
//    private double signedArea(Point[] points) {
//        double sum = 0.0;
//        for (int i = 0; i < points.length-1; i++) {
//            sum = sum + (points[i].x * points[i+1].y) - (points[i].y * points[i+1].x);
//        }
//        return 0.5 * sum;
//    }
//    private Point centroid(Point[] points) {
////        double area=Math.abs(signedArea(points));
////        double cx = 0.0, cy = 0.0;
////        for (int i = 0; i < points.length-1; i++) {
////            cx = cx + (points[i].x + points[i+1].x) * (points[i].y * points[i+1].x - points[i].x * points[i+1].y);
////            cy = cy + (points[i].y + points[i+1].y) * (points[i].y * points[i+1].x - points[i].x * points[i+1].y);
////        }
////        cx /= (6 * area);
////        cy /= (6 * area);
////        return new Point(cx, cy);
//        double cx = 0.0, cy = 0.0, area=0;
//        for (int i = 0; i < points.length-1; i++) {
//            double areaSum =   -(points[i].y * points[i+1].x - points[i].x * points[i+1].y);
//            cx = cx + (points[i].x + points[i+1].x) * areaSum;//(points[i].y * points[i+1].x - points[i].x * points[i+1].y);
//            cy = cy + (points[i].y + points[i+1].y) * areaSum;//(points[i].y * points[i+1].x - points[i].x * points[i+1].y);
//            area += areaSum;
//        }
//        cx /= (6 * 0.5*area);
//        cy /= (6 * 0.5*area);
//
//        return new Point(cx, cy);
//    }
////    /*Replace*/ public void setVertexList(List<GeoNode> mGNList) {
////        mBoundaryVertexList = mGNList;
////        centerLon = 0;
////        centerLat = 0;
////        for (SAILS.GeoNode mGN : mGNList) {
////            if (mGN == mGNList.get(mGNList.size() - 1))
////                break;
////            centerLon += mGN.longitude;
////            centerLat += mGN.latitude;
////        }
////        centerLon /= mGNList.size() - 1;
////        centerLat /= mGNList.size() - 1;
////    }
//
//    static class RegionByLevelComparator implements Comparator<LocationRegion> {
//
//        @Override
//        public int compare(LocationRegion locationRegion, LocationRegion locationRegion2) {
//            if(locationRegion.level<locationRegion2.level)
//                return 1;
//            if(locationRegion.level==locationRegion2.level)
//                return 0;
//            return -1;
//        }
//    }
//}
